@@ -50,6 +50,10 @@
 
 #include "pbd/i18n.h"
 
+#ifdef __APPLE__
+extern void use_cocoa_invalidation (int); // cocoacarbon.mm
+#endif
+
 using namespace Gtk;
 using namespace Gtkmm2ext;
 using namespace ARDOUR;
@@ -498,6 +502,13 @@ ARDOUR_UI::parameter_changed (std::string p)
 		if (UIConfiguration::instance().get_clock_display_limit() > limit) {
 			UIConfiguration::instance().set_clock_display_limit (limit);
 		}
+	} else if (p == "no-strobe") {
+		stop_clocking ();
+		start_clocking ();
+	} else if (p == "use-cocoa-invalidation") {
+#ifdef __APPLE__
+		use_cocoa_invalidation (UIConfiguration::instance().get_use_cocoa_invalidation());
+#endif
 	}
 }
 
