@@ -112,6 +112,7 @@ AutomationLine::AutomationLine (const string&                              name,
 	line = new ArdourCanvas::PolyLine (group);
 	CANVAS_DEBUG_NAME (line, "region gain envelope line");
 	line->set_data ("line", this);
+	line->set_data ("trackview", &trackview);
 	line->set_outline_width (2.0);
 	line->set_covers_threshold (4.0);
 
@@ -209,6 +210,7 @@ AutomationLine::get_uses_gain_mapping () const
 		case BusSendLevel:
 		case EnvelopeAutomation:
 		case TrimAutomation:
+		case SurroundSendLevel:
 		case InsertReturnLevel:
 			return true;
 		default:
@@ -293,7 +295,7 @@ AutomationLine::nth (uint32_t n) const
 }
 
 void
-AutomationLine::modify_point_y (ControlPoint& cp, double y)
+AutomationLine::modify_points_y (std::vector<ControlPoint*> const& cps, double y)
 {
 	/* clamp y-coord appropriately. y is supposed to be a normalized fraction (0.0-1.0),
 	   and needs to be converted to a canvas unit distance.
@@ -307,13 +309,16 @@ AutomationLine::modify_point_y (ControlPoint& cp, double y)
 	trackview.editor().session()->add_command (
 		new MementoCommand<AutomationList> (memento_command_binder(), &get_state(), 0));
 
-	cp.move_to (cp.get_x(), y, ControlPoint::Full);
-
 	alist->freeze ();
-	sync_model_with_view_point (cp);
+	for (auto const& cp : cps) {
+		cp->move_to (cp->get_x(), y, ControlPoint::Full);
+		sync_model_with_view_point (*cp);
+	}
 	alist->thaw ();
 
-	reset_line_coords (cp);
+	for (auto const& cp : cps) {
+		reset_line_coords (*cp);
+	}
 
 	if (line_points.size() > 1) {
 		line->set_steps (line_points, is_stepped());
@@ -400,6 +405,7 @@ AutomationLine::string_to_fraction (string const & s) const
 		case BusSendLevel:
 		case EnvelopeAutomation:
 		case TrimAutomation:
+		case SurroundSendLevel:
 		case InsertReturnLevel:
 			if (s == "-inf") { /* translation */
 				v = 0;
@@ -948,14 +954,13 @@ void
 AutomationLine::get_selectables (timepos_t const & start, timepos_t const & end, double botfrac, double topfrac, list<Selectable*>& results)
 {
 	/* convert fractions to display coordinates with 0 at the top of the track */
-	double const bot_track = (1 - topfrac) * trackview.current_height ();
-	double const top_track = (1 - botfrac) * trackview.current_height ();
+	double const bot_track = (1 - topfrac) * trackview.current_height (); // this should StreamView::child_height () for RegionGain
+	double const top_track = (1 - botfrac) * trackview.current_height (); //  --"--
 
 	for (auto const & cp : control_points) {
 
 		const timepos_t w = session_position ((*cp->model())->when);
 
-
 		if (w >= start && w <= end && cp->get_y() >= bot_track && cp->get_y() <= top_track) {
 			results.push_back (cp);
 		}
@@ -1065,8 +1070,7 @@ AutomationLine::reset_callback (const Evoral::ControlList& events)
 		double ty = model_to_view_coord_y ((*ai)->value);
 
 		if (isnan_local (ty)) {
-			warning << string_compose (_("Ignoring illegal points on AutomationLine \"%1\""),
-			                           _name) << endmsg;
+			warning << string_compose (_("Ignoring illegal points on AutomationLine \"%1\""), _name) << endmsg;
 			continue;
 		}
 
