@@ -459,8 +459,7 @@ lua_exec (std::string cmd)
 	if (x.start()) {
 		return -1;
 	}
-	x.wait ();
-	return 0;
+	return x.wait ();
 }
 #endif
 
@@ -496,15 +495,15 @@ lua_actionlist (lua_State *L)
 		}
 
 		//kinda kludgy way to avoid displaying menu items as mappable
-		if (parts[1] == _("Main_menu"))
+		if (parts[1] == X_("Main_menu"))
 			continue;
-		if (parts[1] == _("JACK"))
+		if (parts[1] == X_("JACK"))
 			continue;
-		if (parts[1] == _("redirectmenu"))
+		if (parts[1] == X_("redirectmenu"))
 			continue;
-		if (parts[1] == _("RegionList"))
+		if (parts[1] == X_("RegionList"))
 			continue;
-		if (parts[1] == _("ProcessorMenu"))
+		if (parts[1] == X_("ProcessorMenu"))
 			continue;
 
 		if (!action_tbl[parts[1]].isTable()) {
@@ -539,6 +538,7 @@ using namespace ARDOUR;
 PBD::Signal0<void> LuaInstance::LuaTimerS;
 PBD::Signal0<void> LuaInstance::LuaTimerDS;
 PBD::Signal0<void> LuaInstance::SetSession;
+PBD::Signal0<void> LuaInstance::SelectionChanged;
 
 void
 LuaInstance::register_hooks (lua_State* L)
@@ -777,7 +777,7 @@ LuaInstance::bind_dialog (lua_State* L)
 }
 
 void
-LuaInstance::register_classes (lua_State* L)
+LuaInstance::register_classes (lua_State* L, bool sandbox)
 {
 	LuaBindings::stddef (L);
 	LuaBindings::common (L);
@@ -911,6 +911,8 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("get_cut_buffer", &PublicEditor::get_cut_buffer)
 		.addRefFunction ("get_selection_extents", &PublicEditor::get_selection_extents)
 
+		.addFunction ("current_mixer_stripable", &PublicEditor::current_mixer_stripable)
+
 		.addFunction ("set_selection", &PublicEditor::set_selection)
 
 		.addFunction ("play_selection", &PublicEditor::play_selection)
@@ -1099,15 +1101,7 @@ LuaInstance::register_classes (lua_State* L)
 
 		.addFunction ("config", &_ui_config)
 
-		.endNamespace () // end ArdourUI
-
-		.beginNamespace ("os")
-#ifndef PLATFORM_WINDOWS
-		.addFunction ("execute", &lua_exec)
-#endif
-		.addCFunction ("forkexec", &lua_forkexec)
-		.endNamespace ();
-
+		.endNamespace (); // end ArdourUI
 	// Editing Symbols
 
 #undef ZOOMFOCUS
@@ -1119,6 +1113,7 @@ LuaInstance::register_classes (lua_State* L)
 #undef IMPORTPOSITION
 #undef IMPORTDISPOSITION
 #undef TEMPOEDITBEHAVIOR
+#undef NOTENAMEDISPLAY
 
 #define ZOOMFOCUS(NAME) .addConst (stringify(NAME), (Editing::ZoomFocus)Editing::NAME)
 #define GRIDTYPE(NAME) .addConst (stringify(NAME), (Editing::GridType)Editing::NAME)
@@ -1128,11 +1123,21 @@ LuaInstance::register_classes (lua_State* L)
 #define IMPORTMODE(NAME) .addConst (stringify(NAME), (Editing::ImportMode)Editing::NAME)
 #define IMPORTPOSITION(NAME) .addConst (stringify(NAME), (Editing::ImportPosition)Editing::NAME)
 #define IMPORTDISPOSITION(NAME) .addConst (stringify(NAME), (Editing::ImportDisposition)Editing::NAME)
-#define TEMPOEDITBEHAVIOR(NAME) .addConst (stringify(NAME), (Editing::TempoEditBehavior)Editing::NAME)
+#define NOTENAMEDISPLAY(NAME) .addConst (stringify(NAME), (Editing::NoteNameDisplay)Editing::NAME)
 	luabridge::getGlobalNamespace (L)
 		.beginNamespace ("Editing")
 #		include "editing_syms.h"
 		.endNamespace ();
+
+	if (!sandbox) {
+		luabridge::getGlobalNamespace (L)
+			.beginNamespace ("os")
+#ifndef PLATFORM_WINDOWS
+			.addFunction ("execute", &lua_exec)
+#endif
+			.addCFunction ("forkexec", &lua_forkexec)
+			.endNamespace ();
+	}
 }
 
 #undef xstr
@@ -1172,6 +1177,7 @@ LuaInstance::destroy_instance ()
 }
 
 LuaInstance::LuaInstance ()
+	: lua (true, UIConfiguration::instance().get_sandbox_all_lua_scripts ())
 {
 	lua.Print.connect (&_lua_print);
 	init ();
@@ -1191,10 +1197,27 @@ LuaInstance::~LuaInstance ()
 	_callbacks.clear();
 }
 
+static std::string
+lua_read_script (std::string const& fn)
+{
+	if (!UIConfiguration::instance().get_update_action_scripts ()) {
+		return "";
+	}
+	try {
+		return Glib::file_get_contents (fn);
+	} catch (...) { }
+	return "";
+}
+
 void
 LuaInstance::init ()
 {
-	lua.sandbox (false);
+	luabridge::getGlobalNamespace (lua.getState())
+		.beginNamespace ("Internal")
+		.addFunction ("get_factory_bytecode", &LuaScripting::get_factory_bytecode)
+		.addFunction ("read_script", &lua_read_script)
+		.endNamespace ();
+
 	lua.do_command (
 			"function ScriptManager ()"
 			"  local self = { scripts = {}, instances = {}, icons = {} }"
@@ -1297,10 +1320,20 @@ LuaInstance::init ()
 			"   collectgarbage()"
 			"  end"
 			""
+			"  local get_factory_bytecode = Internal.get_factory_bytecode"
+			"  local read_script = Internal.read_script"
 			"  local restore = function (state)"
 			"   clear()"
 			"   load (state)()"
 			"   for i, s in pairs (scripts) do"
+			"    if s['a']['x-script-origin'] then"
+			"       local sc = read_script (s['a']['x-script-origin'])"
+			"       if sc ~= '' then"
+			"         fnc = nil load (get_factory_bytecode (sc, 'factory', 'fnc'))()"
+			"         icn = nil load (get_factory_bytecode (sc, 'icon', 'icn'))()"
+			"         if fnc ~= '' and type(fnc) == 'string' then s['f'] = fnc s['c'] = icn s['s'] = sc end "
+			"       end"
+			"    end"
 			"    addinternal (i, s['n'], s['s'], load(s['f']), type (s['c']) ~= \"string\" or s['c'] == '' or load (s['c']), s['a'])"
 			"   end"
 			"   collectgarbage()"
@@ -1318,6 +1351,7 @@ LuaInstance::init ()
 	try {
 		luabridge::LuaRef lua_mgr = luabridge::getGlobal (L, "manager");
 		lua.do_command ("manager = nil"); // hide it.
+		lua.do_command ("Internal = nil");
 		lua.do_command ("collectgarbage()");
 
 		_lua_add_action = new luabridge::LuaRef(lua_mgr["add"]);
@@ -1341,11 +1375,17 @@ LuaInstance::init ()
 		abort(); /*NOTREACHED*/
 	}
 
-	register_classes (L);
+	LuaInstance::register_classes (L, UIConfiguration::instance().get_sandbox_all_lua_scripts ());
 	register_hooks (L);
 
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
+
+	Selection& sel (PublicEditor::instance().get_selection ());
+	sel.TimeChanged.connect (sigc::mem_fun (*this, &LuaInstance::selection_changed));
+	sel.RegionsChanged.connect (sigc::mem_fun (*this, &LuaInstance::selection_changed));
+	sel.TracksChanged.connect (sigc::mem_fun (*this, &LuaInstance::selection_changed));
+	sel.MarkersChanged.connect (sigc::mem_fun (*this, &LuaInstance::selection_changed));
 }
 
 int
@@ -1422,7 +1462,7 @@ void LuaInstance::set_session (Session* s)
 	for (LuaCallbackMap::iterator i = _callbacks.begin(); i != _callbacks.end(); ++i) {
 		i->second->set_session (s);
 	}
-	second_connection = Timers::rapid_connect (sigc::mem_fun(*this, & LuaInstance::every_second));
+	second_connection = Timers::second_connect (sigc::mem_fun(*this, & LuaInstance::every_second));
 	point_one_second_connection = Timers::rapid_connect (sigc::mem_fun(*this, & LuaInstance::every_point_one_seconds));
 	SetSession (); /* EMIT SIGNAL */
 }
@@ -1458,6 +1498,12 @@ LuaInstance::every_point_one_seconds ()
 	LuaTimerDS (); // emit signal
 }
 
+void
+LuaInstance::selection_changed ()
+{
+	SelectionChanged (); // emit signal
+}
+
 int
 LuaInstance::set_state (const XMLNode& node)
 {
@@ -1513,8 +1559,8 @@ LuaInstance::pre_seed_script (std::string const& name, int& id)
 	if (spi) {
 		try {
 			std::string script = Glib::file_get_contents (spi->path);
-			LuaState ls;
-			register_classes (ls.getState ());
+			LuaState ls (true, true);
+			register_classes (ls.getState (), true);
 			LuaScriptParamList lsp = LuaScriptParams::script_params (ls, spi->path, "action_params");
 			LuaScriptParamPtr lspp (new LuaScriptParam("x-script-origin", "", spi->path, false, true));
 			lsp.push_back (lspp);
@@ -1581,8 +1627,8 @@ LuaInstance::interactive_add (Gtk::Window& parent, LuaScriptInfo::ScriptType typ
 		return false;
 	}
 
-	LuaState ls;
-	register_classes (ls.getState ());
+	LuaState ls (true, true);
+	register_classes (ls.getState (), true);
 	LuaScriptParamList lsp = LuaScriptParams::script_params (ls, spi->path, param_function);
 
 	/* allow cancel */
@@ -1594,7 +1640,7 @@ LuaInstance::interactive_add (Gtk::Window& parent, LuaScriptInfo::ScriptType typ
 
 	ScriptParameterDialog spd (_("Set Script Parameters"), spi, reg, lsp);
 
-	if (spd.need_interation ()) {
+	if (spd.need_interaction ()) {
 		switch (spd.run ()) {
 			case Gtk::RESPONSE_ACCEPT:
 				break;
@@ -1851,9 +1897,8 @@ LuaInstance::register_lua_slot (const std::string& name, const std::string& scri
 	/* parse script, get ActionHook(s) from script */
 	ActionHook ah;
 	try {
-		LuaState l;
+		LuaState l (true, true);
 		l.Print.connect (&_lua_print);
-		l.sandbox (true);
 		lua_State* L = l.getState();
 		register_hooks (L);
 		l.do_command ("function ardour () end");
@@ -1958,6 +2003,7 @@ LuaCallback::LuaCallback (Session *s,
 		const ActionHook& ah,
 		const ARDOUR::LuaScriptParamList& args)
 	: SessionHandlePtr (s)
+	, lua (true, UIConfiguration::instance().get_sandbox_all_lua_scripts ())
 	, _id ("0")
 	, _name (name)
 	, _signals (ah)
@@ -1991,6 +2037,7 @@ LuaCallback::LuaCallback (Session *s,
 
 LuaCallback::LuaCallback (Session *s, XMLNode & node)
 	: SessionHandlePtr (s)
+	, lua (true, UIConfiguration::instance().get_sandbox_all_lua_scripts ())
 {
 	XMLNode* child = NULL;
 	if (node.name() != X_("LuaCallback")
@@ -2074,7 +2121,6 @@ void
 LuaCallback::init (void)
 {
 	lua.Print.connect (&_lua_print);
-	lua.sandbox (false);
 
 	lua.do_command (
 			"function ScriptManager ()"
@@ -2195,7 +2241,7 @@ LuaCallback::init (void)
 		abort(); /*NOTREACHED*/
 	}
 
-	LuaInstance::register_classes (L);
+	LuaInstance::register_classes (L, UIConfiguration::instance().get_sandbox_all_lua_scripts ());
 	LuaInstance::register_hooks (L);
 
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
