@@ -64,6 +64,11 @@
 #include "editing.h"
 #include "selection.h"
 
+namespace Temporal {
+	class TempoPoint;
+	class MeterPoint;
+}
+
 namespace ARDOUR {
 	class Session;
 	class Region;
@@ -237,6 +242,8 @@ public:
 	virtual void set_selection (std::list<Selectable*>, Selection::Operation) = 0;
 	virtual void set_selected_midi_region_view (MidiRegionView&) = 0;
 
+	virtual std::shared_ptr<ARDOUR::Route> current_mixer_stripable () const = 0;
+
 	virtual bool extend_selection_to_track (TimeAxisView&) = 0;
 	virtual void play_solo_selection(bool restart) = 0;
 	virtual void play_selection () = 0;
@@ -305,6 +312,8 @@ public:
 	/** Open Simple Export Dialog */
 	virtual void quick_export () = 0;
 
+	virtual void surround_export () = 0;
+
 	virtual void loudness_assistant (bool) = 0;
 
 	virtual void register_actions () = 0;
@@ -450,6 +459,8 @@ public:
 
 	virtual bool canvas_scroll_event (GdkEventScroll* event, bool from_canvas) = 0;
 	virtual bool canvas_control_point_event (GdkEvent* event, ArdourCanvas::Item*, ControlPoint*) = 0;
+	virtual bool canvas_velocity_event (GdkEvent* event, ArdourCanvas::Item*) = 0;
+	virtual bool canvas_velocity_base_event (GdkEvent* event, ArdourCanvas::Item*) = 0;
 	virtual bool canvas_line_event (GdkEvent* event, ArdourCanvas::Item*, AutomationLine*) = 0;
 	virtual bool canvas_selection_rect_event (GdkEvent* event, ArdourCanvas::Item*, SelectionRect*) = 0;
 	virtual bool canvas_selection_start_trim_event (GdkEvent* event, ArdourCanvas::Item*, SelectionRect*) = 0;
@@ -521,7 +532,7 @@ public:
 		_commit_tempo_map_edit (map, with_update);
 	}
 
-	virtual Temporal::TempoMap::WritableSharedPtr begin_tempo_mapping () = 0;
+	virtual Temporal::TempoMap::WritableSharedPtr begin_tempo_mapping (Temporal::DomainBounceInfo&) = 0;
 	virtual void abort_tempo_mapping () = 0;
 	virtual void commit_tempo_mapping (Temporal::TempoMap::WritableSharedPtr& map) = 0;
 
@@ -542,11 +553,14 @@ public:
 
 	virtual ARDOUR::Location* find_location_from_marker (ArdourMarker*, bool&) const = 0;
 	virtual ArdourMarker* find_marker_from_location_id (PBD::ID const&, bool) const = 0;
+	virtual TempoMarker* find_marker_for_tempo (Temporal::TempoPoint const &) = 0;
+	virtual MeterMarker* find_marker_for_meter (Temporal::MeterPoint const &) = 0;
 
 	virtual void snap_to_with_modifier (Temporal::timepos_t & first,
 	                                    GdkEvent const*      ev,
 	                                    Temporal::RoundMode    direction = Temporal::RoundNearest,
-	                                    ARDOUR::SnapPref     gpref = ARDOUR::SnapToAny_Visual) = 0;
+	                                    ARDOUR::SnapPref     gpref = ARDOUR::SnapToAny_Visual,
+	                                    bool ensure_snap = false) = 0;
 	virtual Temporal::timepos_t snap_to_bbt (Temporal::timepos_t const & pos, Temporal::RoundMode, ARDOUR::SnapPref) = 0;
 
 	virtual void set_snapped_cursor_position (Temporal::timepos_t const & pos) = 0;
@@ -569,9 +583,11 @@ public:
 
 	virtual void queue_redisplay_track_views () = 0;
 
-	virtual ARDOUR::Quantize get_quantize_op (bool force_dialog, bool& did_show_dialog) = 0;
+	virtual ARDOUR::Quantize* get_quantize_op () = 0;
 	virtual void apply_midi_note_edit_op (ARDOUR::MidiOperator& op, const RegionSelection& rs) = 0;
 
+	virtual void set_tempo_curve_range (double& max, double& min) const = 0;
+
 	/// Singleton instance, set up by Editor::Editor()
 
 	static PublicEditor* _instance;
