@@ -173,8 +173,19 @@ enum AutomationType {
 	MonitoringAutomation,
 	BusSendLevel,
 	BusSendEnable,
+	SurroundSendLevel,
 	InsertReturnLevel,
 	MainOutVolume,
+	MidiVelocityAutomation,
+	PanSurroundX,
+	PanSurroundY,
+	PanSurroundZ,
+	PanSurroundSize,
+	PanSurroundSnap,
+	BinauralRenderMode,
+	PanSurroundElevationEnable,
+	PanSurroundZones,
+	PanSurroundRamp,
 
 	/* used only by Controllable Descriptor to access send parameters */
 
@@ -434,6 +445,12 @@ enum EditMode {
 	Lock
 };
 
+enum SnapTarget {
+	SnapTargetGrid,
+	SnapTargetOther,
+	SnapTargetBoth
+};
+
 enum RippleMode {
 	RippleSelected,
 	RippleAll,
@@ -654,9 +671,9 @@ typedef std::list<std::shared_ptr<GraphNode> > GraphNodeList;
 typedef std::list<std::shared_ptr<Stripable> > StripableList;
 typedef std::list<std::weak_ptr  <Route> > WeakRouteList;
 typedef std::list<std::weak_ptr  <Stripable> > WeakStripableList;
-typedef std::list<std::shared_ptr<AutomationControl> > ControlList;
-typedef std::list<std::weak_ptr  <AutomationControl> > WeakControlList;
-typedef std::list<std::shared_ptr<SlavableAutomationControl> > SlavableControlList;
+typedef std::list<std::shared_ptr<AutomationControl> > AutomationControlList;
+typedef std::list<std::weak_ptr  <AutomationControl> > WeakAutomationControlList;
+typedef std::list<std::shared_ptr<SlavableAutomationControl> > SlavableAutomationControlList;
 typedef std::set <AutomationType> AutomationTypeSet;
 
 typedef std::list<std::shared_ptr<VCA> > VCAList;
@@ -718,11 +735,12 @@ enum AppleNSGLViewMode {
  */
 struct RouteProcessorChange {
 	enum Type {
-		MeterPointChange = 0x1,
-		RealTimeChange   = 0x2,
-		GeneralChange    = 0x4,
-		SendReturnChange = 0x8,
-		CustomPinChange  = 0x10
+		NoProcessorChange = 0x0,
+		MeterPointChange  = 0x1,
+		RealTimeChange    = 0x2,
+		GeneralChange     = 0x4,
+		SendReturnChange  = 0x8,
+		CustomPinChange   = 0x10
 	};
 
 	RouteProcessorChange () : type (GeneralChange), meter_visibly_changed (true)
@@ -951,7 +969,6 @@ struct ProcessedRanges {
 	ProcessedRanges() : start { 0, 0 }, end { 0, 0 }, cnt (0) {}
 };
 
-
 } // namespace ARDOUR
 
 /* for now, break the rules and use "using" to make this "global" */
