@@ -57,14 +57,16 @@ DiskReader::Declicker DiskReader::loop_declick_in;
 DiskReader::Declicker DiskReader::loop_declick_out;
 samplecnt_t           DiskReader::loop_fade_length (0);
 
-DiskReader::DiskReader (Session& s, Track& t, string const& str, Temporal::TimeDomain td, DiskIOProcessor::Flag f)
-	: DiskIOProcessor (s, t, X_("player:") + str, f, td)
+DiskReader::DiskReader (Session& s, Track& t, string const& str, Temporal::TimeDomainProvider const & tdp, DiskIOProcessor::Flag f)
+	: DiskIOProcessor (s, t, X_("player:") + str, f, tdp)
 	, overwrite_sample (0)
 	, run_must_resolve (false)
 	, _declick_amp (s.nominal_sample_rate ())
 	, _declick_offs (0)
 	, _declick_enabled (false)
 	, last_refill_loop_start (0)
+	, _midi_catchup (false)
+	, _need_midi_catchup (false)
 {
 	file_sample[DataType::AUDIO] = 0;
 	file_sample[DataType::MIDI]  = 0;
@@ -708,6 +710,7 @@ DiskReader::overwrite_existing_audio ()
 		chunk2_cnt = to_overwrite - chunk1_cnt;
 	}
 
+	boost::scoped_array<Sample> sum_buffer (new Sample[to_overwrite]);
 	boost::scoped_array<Sample> mixdown_buffer (new Sample[to_overwrite]);
 	boost::scoped_array<float>  gain_buffer (new float[to_overwrite]);
 	uint32_t                    n   = 0;
@@ -725,18 +728,21 @@ DiskReader::overwrite_existing_audio ()
 		start = overwrite_sample;
 
 		if (chunk1_cnt) {
-			if (audio_read (buf + chunk1_offset, mixdown_buffer.get (), gain_buffer.get (), start, chunk1_cnt, rci, n, reversed) != (samplecnt_t)chunk1_cnt) {
+			if (audio_read (sum_buffer.get (), mixdown_buffer.get (), gain_buffer.get (), start, chunk1_cnt, rci, n, reversed) != (samplecnt_t)chunk1_cnt) {
 				error << string_compose (_("DiskReader %1: when overwriting(1), cannot read %2 from playlist at sample %3"), id (), chunk1_cnt, overwrite_sample) << endmsg;
 				ret = false;
+				++n;
 				continue;
 			}
+			memcpy (buf + chunk1_offset, sum_buffer.get (), sizeof (float) * chunk1_cnt);
 		}
 
 		if (chunk2_cnt) {
-			if (audio_read (buf, mixdown_buffer.get (), gain_buffer.get (), start, chunk2_cnt, rci, n, reversed) != (samplecnt_t)chunk2_cnt) {
+			if (audio_read (sum_buffer.get (), mixdown_buffer.get (), gain_buffer.get (), start, chunk2_cnt, rci, n, reversed) != (samplecnt_t)chunk2_cnt) {
 				error << string_compose (_("DiskReader %1: when overwriting(2), cannot read %2 from playlist at sample %3"), id (), chunk2_cnt, overwrite_sample) << endmsg;
 				ret = false;
 			}
+			memcpy (buf, sum_buffer.get (), sizeof (float) * chunk2_cnt);
 		}
 
 		if (!rci->initialized) {
@@ -745,6 +751,7 @@ DiskReader::overwrite_existing_audio ()
 				rci->initialized = true;
 			}
 		}
+		++n;
 	}
 
 	file_sample[DataType::AUDIO] = start;
@@ -1482,6 +1489,14 @@ DiskReader::get_midi_playback (MidiBuffer& dst, samplepos_t start_sample, sample
 				effective_end   = min (effective_start + cnt, loop_end);
 				assert (effective_end > effective_start);
 
+
+				if (_midi_catchup && _need_midi_catchup) {
+					MidiStateTracker mst;
+					rtmb->track (mst, effective_start, effective_end);
+					mst.flush (dst, 0, false);
+					_need_midi_catchup = false;
+				}
+
 				const samplecnt_t this_read = effective_end - effective_start;
 
 				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer LOOP read, from %1 to %2 (%3)\n", effective_start, effective_end, this_read));
@@ -1489,7 +1504,7 @@ DiskReader::get_midi_playback (MidiBuffer& dst, samplepos_t start_sample, sample
 #ifndef NDEBUG
 				size_t events_read =
 #endif
-				    rtmb->read (*target, effective_start, effective_end, _tracker, offset);
+					rtmb->read (*target, effective_start, effective_end, _tracker, offset);
 
 				cnt -= this_read;
 				effective_start += this_read;
@@ -1509,6 +1524,12 @@ DiskReader::get_midi_playback (MidiBuffer& dst, samplepos_t start_sample, sample
 
 		} else {
 			DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer read, from %1 to %2 (%3)\n", start_sample, end_sample, nframes));
+			if (_midi_catchup && _need_midi_catchup) {
+				MidiStateTracker mst;
+				rtmb->track (mst, start_sample, end_sample);
+				mst.flush (dst, 0, false);
+				_need_midi_catchup = false;
+			}
 			DEBUG_RESULT (size_t, events_read, rtmb->read (*target, start_sample, end_sample, _tracker));
 			DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("%1 MDS events read %2 range %3 .. %4\n", _name, events_read, playback_sample, playback_sample + nframes));
 		}
@@ -1955,3 +1976,9 @@ DiskReader::setup_preloop_buffer ()
 		++channel;
 	}
 }
+
+void
+DiskReader::set_need_midi_catchup (bool yn)
+{
+	_need_midi_catchup = yn;
+}
