@@ -167,8 +167,10 @@ float ARDOUR::ui_scale_factor = 1.0;
 
 static bool have_old_configuration_files = false;
 static bool running_from_gui             = false;
-static int  cpu_dma_latency_fd           = -1;
 
+#if !(defined PLATFORM_WINDOWS || defined __APPLE__)
+static int  cpu_dma_latency_fd           = -1;
+#endif
 
 namespace ARDOUR {
 extern void setup_enum_writer ();
@@ -422,12 +424,19 @@ copy_configuration_files (string const& old_dir, string const& new_dir, int old_
 
 		copy_file (old_name, new_name);
 
+		old_name = Glib::build_filename (old_dir, X_("recent_templates"));
+		new_name = Glib::build_filename (new_dir, X_("recent_templates"));
+
+		copy_file (old_name, new_name);
+
 		old_name = Glib::build_filename (old_dir, X_("sfdb"));
 		new_name = Glib::build_filename (new_dir, X_("sfdb"));
 
 		copy_file (old_name, new_name);
 
-		/* can only copy ardour.rc/config - UI config is not compatible */
+		/* can only copy ardour.rc/config unconditionally, there are
+		 * issues with old ui_config versions.
+		 */
 
 		/* users who have been using git/nightlies since the last
 		 * release of 3.5 will have $CONFIG/config rather than
@@ -435,7 +444,7 @@ copy_configuration_files (string const& old_dir, string const& new_dir, int old_
 		 * to avoid confusion.
 		 */
 
-		string old_name = Glib::build_filename (old_dir, X_("config"));
+		old_name = Glib::build_filename (old_dir, X_("config"));
 
 		if (!Glib::file_test (old_name, Glib::FILE_TEST_EXISTS)) {
 			old_name = Glib::build_filename (old_dir, X_("ardour.rc"));
@@ -445,6 +454,13 @@ copy_configuration_files (string const& old_dir, string const& new_dir, int old_
 
 		copy_file (old_name, new_name);
 
+		/* default Session Properties */
+
+		old_name = Glib::build_filename (old_dir, X_("session.rc"));
+		new_name = Glib::build_filename (new_dir, X_("session.rc"));
+
+		copy_file (old_name, new_name);
+
 		/* copy templates and route templates */
 
 		old_name = Glib::build_filename (old_dir, X_("templates"));
@@ -457,7 +473,7 @@ copy_configuration_files (string const& old_dir, string const& new_dir, int old_
 
 		copy_recurse (old_name, new_name);
 
-		/* presets */
+		/* plugin presets (VST2, Lua) */
 
 		old_name = Glib::build_filename (old_dir, X_("presets"));
 		new_name = Glib::build_filename (new_dir, X_("presets"));
@@ -481,21 +497,72 @@ copy_configuration_files (string const& old_dir, string const& new_dir, int old_
 
 		copy_file (old_name, new_name);
 
-		/* export formats */
+
+		/* export formats and presets */
 
 		old_name = Glib::build_filename (old_dir, export_formats_dir_name);
 		new_name = Glib::build_filename (new_dir, export_formats_dir_name);
 
-		vector<string> export_formats;
+		vector<string> export_settings;
 		g_mkdir_with_parents (Glib::build_filename (new_dir, export_formats_dir_name).c_str (), 0755);
-		find_files_matching_pattern (export_formats, old_name, X_("*.format"));
-		for (vector<string>::iterator i = export_formats.begin (); i != export_formats.end (); ++i) {
-			std::string from = *i;
-			std::string to   = Glib::build_filename (new_name, Glib::path_get_basename (*i));
+		find_files_matching_pattern (export_settings, old_name, X_("*.format"));
+		find_files_matching_pattern (export_settings, old_name, X_("*.preset"));
+		for (auto const& from: export_settings) {
+			std::string to   = Glib::build_filename (new_name, Glib::path_get_basename (from));
 			copy_file (from, to);
 		}
 	}
 
+	if (old_version >= 7) {
+		/* Lua scripts  - older scripts are no longer compatible */
+		old_name = Glib::build_filename (old_dir, X_("scripts"));
+		new_name = Glib::build_filename (new_dir, X_("scripts"));
+		copy_recurse (old_name, new_name);
+
+		old_name = Glib::build_filename (old_dir, X_("ui_scripts"));
+		new_name = Glib::build_filename (new_dir, X_("ui_scripts"));
+		copy_file (old_name, new_name);
+
+		old_name = Glib::build_filename (old_dir, X_("luahist"));
+		new_name = Glib::build_filename (new_dir, X_("luahist"));
+		copy_file (old_name, new_name);
+
+		/* Port Metadata (since v7.0) */
+		old_name = Glib::build_filename (old_dir, X_("port_metadata"));
+		new_name = Glib::build_filename (new_dir, X_("port_metadata"));
+		copy_file (old_name, new_name);
+
+		/* UIConfig v7 compatible */
+		old_name = Glib::build_filename (old_dir, X_("ui_config"));
+		new_name = Glib::build_filename (new_dir, X_("ui_config"));
+		copy_file (old_name, new_name);
+	}
+
+	return 0;
+}
+
+static int
+copy_cache_files (string const& old_dir, string const& new_dir, int old_version)
+{
+	if (g_mkdir_with_parents (new_dir.c_str (), 0755)) {
+		return -1;
+	}
+	/* since v7 plugin cache files are versioned */
+	if (old_version < 7) {
+		return 0;
+	}
+
+	/* copy complete cache:
+	 * - blacklist files
+	 * - vst/.*v2i (VST2 Cache - Intel)
+	 * - vst/.*v3i (VST3 Cache - Intel or Apple/Rosetta)
+	 * - vst-arm64/.*v3i (Apple/ARM native VST3 Cache)
+	 * - auv2/.*a3i (Audio Unit Cache)
+	 * - vst*_blacklist.txt
+	 * - auv2_*blacklist.txt
+	 */
+	copy_recurse (old_dir, new_dir, true);
+
 	return 0;
 }
 
@@ -530,9 +597,12 @@ ARDOUR::handle_old_configuration_files (boost::function<bool(std::string const&,
 		int    old_version        = current_version - 1;
 		string old_config_dir     = user_config_directory (old_version);
 		string current_config_dir = user_config_directory (current_version);
+		string old_cache_dir      = user_cache_directory (old_version);
+		string current_cache_dir  = user_cache_directory (current_version);
 
 		if (ui_handler (old_config_dir, current_config_dir, old_version)) {
 			copy_configuration_files (old_config_dir, current_config_dir, old_version);
+			copy_cache_files (old_cache_dir, current_cache_dir, old_version);
 			return 1;
 		}
 	}
@@ -680,6 +750,7 @@ ARDOUR::init (bool try_optimization, const char* localedir, bool with_gui)
 
 	reserved_io_names[_("Monitor")]             = true;
 	reserved_io_names[_("Master")]              = true;
+	reserved_io_names[_("Surround")]            = true;
 	reserved_io_names[X_("auditioner")]         = true; // auditioner.cc  Track (s, "auditioner",...)
 	reserved_io_names[X_("x-virtual-keyboard")] = false;
 	reserved_io_names[X_("MIDI Tracer 1")]      = false;
@@ -857,6 +928,9 @@ ARDOUR::setup_fpu ()
 		    : "=r"(cw)::"memory");
 	}
 
+#elif defined(__ARMEL__)
+	/* no FTZ instructions on that platform */
+#warning you do not want to compile Arodur on armel.
 #elif defined(__arm__)
 	/* http://infocenter.arm.com/help/topic/com.arm.doc.dui0068b/BCFHFBGA.html
 	 * bit 24: flush-to-zero */
@@ -875,7 +949,13 @@ ARDOUR::setup_fpu ()
 /* this can be changed to modify the translation behaviour for
    cases where the user has never expressed a preference.
 */
+
+
+#if defined(PLATFORM_WINDOWS) || defined(__APPLE__)
+static const bool translate_by_default = false;
+#else
 static const bool translate_by_default = true;
+#endif
 
 string
 ARDOUR::translation_enable_path ()
