@@ -306,8 +306,8 @@ IO::add_port (string destination, void* src, DataType type)
 	ChanCount after = before;
 	after.set (type, after.get (type) + 1);
 
-	bool const r = PortCountChanging (after); /* EMIT SIGNAL */
-	if (r) {
+	boost::optional<bool> const r = PortCountChanging (after); /* EMIT SIGNAL */
+	if (r.value_or (false)) {
 		return -1;
 	}
 
@@ -1159,7 +1159,7 @@ IO::apply_pretty_name ()
 	for (PortSet::iterator i = _ports.begin (); i != _ports.end(); ++i, ++pn) {
 		(*i)->set_pretty_name (string_compose (("%1/%2 %3"),
 					_pretty_name_prefix,
-					_direction == Output ? _("Out") : _("In"),
+					_direction == Output ? S_("IO|Out") : S_("IO|In"),
 					pn));
 	}
 }
@@ -1536,6 +1536,10 @@ IO::bundle_channel_name (uint32_t c, uint32_t n, DataType t) const
 
 	if (t == DataType::AUDIO) {
 
+		if (n == _audio_channel_names.size () && c < _audio_channel_names.size ()) {
+			return _audio_channel_names.at (c);
+		}
+
 		switch (n) {
 		case 1:
 			return _("mono");
@@ -1688,9 +1692,13 @@ IO::copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, samplecn
 		port_buffer.read_from (*prev, nframes, offset);
 		++o;
 	}
+}
 
+void
+IO::flush_buffers (pframes_t nframes)
+{
 	/* when port is both externally and internally connected,
-	 * make data directly available to downstream internal ports */
+	 * make data available to downstream internal ports */
 	for (auto const& p : _ports) {
 		p->flush_buffers (nframes);
 	}
