@@ -18,6 +18,7 @@
 #include <cassert>
 
 #include "pbd/types_convert.h"
+#include "pbd/unwind.h"
 #include "pbd/xml++.h"
 
 #include "temporal/tempo.h"
@@ -46,6 +47,7 @@ IOPlug::IOPlug (Session& s, std::shared_ptr<Plugin> p, bool pre)
 	, _plugin (p)
 	, _pre (pre)
 	, _plugin_signal_latency (0)
+	, _configuring_io (false)
 	, _window_proxy (0)
 {
 	_stat_reset.store (0);
@@ -57,6 +59,12 @@ IOPlug::IOPlug (Session& s, std::shared_ptr<Plugin> p, bool pre)
 	}
 	_input.reset (new IO (_session, io_name (), IO::Input));
 	_output.reset (new IO (_session, io_name (), IO::Output));
+
+	/* do not allow to add/remove ports (for now).
+	 * when adding ports _buf will needs to be resized.
+	 */
+	_input->PortCountChanging.connect_same_thread (*this, [this](ChanCount) { return !_configuring_io; });
+	_output->PortCountChanging.connect_same_thread (*this, [this](ChanCount) { return !_configuring_io; });
 }
 
 IOPlug::~IOPlug ()
@@ -64,12 +72,18 @@ IOPlug::~IOPlug ()
 	for (CtrlOutMap::const_iterator i = _control_outputs.begin(); i != _control_outputs.end(); ++i) {
 		std::dynamic_pointer_cast<ReadOnlyControl>(i->second)->drop_references ();
 	}
+
+	Glib::Threads::Mutex::Lock lm (_control_lock);
+	for (Controls::const_iterator li = _controls.begin(); li != _controls.end(); ++li) {
+		std::dynamic_pointer_cast<AutomationControl>(li->second)->drop_references ();
+	}
+	_controls.clear ();
 }
 
 std::string
 IOPlug::io_name (std::string const& n) const
 {
-	return (string_compose ("%1/%2/%3", _("IO"), _pre ? _("Pre"): _("Post"), n.empty () ? name () : n));
+	return (string_compose ("%1/%2/%3", _("IO"), _pre ? S_("IO|Pre"): S_("IO|Post"), n.empty () ? name () : n));
 }
 
 std::string
@@ -374,6 +388,8 @@ IOPlug::ui_elements () const
 bool
 IOPlug::ensure_io ()
 {
+	PBD::Unwinder<bool> uw (_configuring_io, true);
+
 	/* must be called with process-lock held */
 	if (_input->ensure_io (_n_in, false, this) != 0) {
 		return false;
@@ -489,9 +505,7 @@ IOPlug::connect_and_run (samplepos_t start, pframes_t n_samples)
 			}
 		}
 	}
-	for (PortSet::iterator i = ports.begin(); i != ports.end(); ++i) {
-		i->flush_buffers (n_samples);
-	}
+	_output->flush_buffers (n_samples);;
 
 	const samplecnt_t l = effective_latency ();
 	if (_plugin_signal_latency != l) {
@@ -594,6 +608,56 @@ IOPlug::direct_feeds_according_to_reality (std::shared_ptr<GraphNode> node, bool
 
 /* ****************************************************************************/
 
+bool
+IOPlug::can_reset_all_parameters ()
+{
+	bool all = true;
+	uint32_t params = 0;
+	for (uint32_t par = 0; par < _plugin->parameter_count(); ++par) {
+		bool ok=false;
+		const uint32_t cid = _plugin->nth_parameter (par, ok);
+
+		if (!ok || !_plugin->parameter_is_input(cid)) {
+			continue;
+		}
+
+		++params;
+	}
+	return all && (params > 0);
+}
+
+bool
+IOPlug::reset_parameters_to_default ()
+{
+	bool all = true;
+
+	for (uint32_t par = 0; par < _plugin->parameter_count(); ++par) {
+		bool ok=false;
+		const uint32_t cid = _plugin->nth_parameter (par, ok);
+
+		if (!ok || !_plugin->parameter_is_input(cid)) {
+			continue;
+		}
+
+		const float dflt = _plugin->default_value (cid);
+		const float curr = _plugin->get_parameter (cid);
+
+		if (dflt == curr) {
+			continue;
+		}
+
+		std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl>(control (Evoral::Parameter(PluginAutomation, 0, cid)));
+		if (!ac) {
+			continue;
+		}
+
+		ac->set_value (dflt, Controllable::NoGroup);
+	}
+	return all;
+}
+
+/* ****************************************************************************/
+
 IOPlug::PluginControl::PluginControl (IOPlug*                     p,
                                       Evoral::Parameter const&    param,
                                       ParameterDescriptor const&  desc)
